@@ -462,8 +462,6 @@ def plot_occupancy_grid(
         x_limits=None,
         y_limits=None,
         z_limits=None,
-        step_x=0.01,
-        step_y=0.01,
         n_processes=None,
         fig_size=(5, 5),
         fig_size_unit='cm',
@@ -536,8 +534,6 @@ def plot_occupancy_grid(
         x_limits=x_limits,
         y_limits=y_limits,
         z_limits=z_limits,
-        step_x=step_x,
-        step_y=step_y,
         n_processes=n_processes,
         mesh_type=mesh_type,
         ground_plane_models=ground_plane_models)
@@ -576,19 +572,15 @@ def plot_occupancy_grid(
     max_axis_y = None
 
     if occupancy_output is not None:
-        if with_ground_plane and len(occupancy_output['ground_plane']) > 0:
-            ax.patch.set_facecolor(unavailable_color)
-            PCG_ROOT_LOGGER.info(
-                'Adding ground plane models as '
-                'free-space in the occupancy grid')
+        if with_ground_plane:
+            if occupancy_output['ground_plane'] is not None:
+                ax.patch.set_facecolor(unavailable_color)
+                PCG_ROOT_LOGGER.info(
+                    'Adding ground plane models as '
+                    'free-space in the occupancy grid')
 
-            for tag in occupancy_output['ground_plane']:
-                if is_excluded(tag):
-                    PCG_ROOT_LOGGER.info(
-                        'Model {} is excluded from final map'.format(tag))
-                    continue
                 patch = descartes.PolygonPatch(
-                    occupancy_output['ground_plane'][tag],
+                    occupancy_output['ground_plane'],
                     facecolor=free_color,
                     edgecolor=free_color,
                     alpha=1,
@@ -596,42 +588,46 @@ def plot_occupancy_grid(
                 ax.add_patch(patch)
 
                 PCG_ROOT_LOGGER.info(
-                    'Ground-plane model <{}> added'
-                    ' to occupancy grid'.format(tag))
+                    'Ground-plane footprint added'
+                    ' to occupancy grid')
 
                 if axis_x_limits is None:
                     if min_axis_x is None:
                         min_axis_x = \
-                            occupancy_output['ground_plane'][tag].bounds[0]
+                            occupancy_output['ground_plane'].bounds[0]
                     else:
                         min_axis_x = min(
                             min_axis_x,
-                            occupancy_output['ground_plane'][tag].bounds[0])
+                            occupancy_output['ground_plane'].bounds[0])
 
                     if max_axis_x is None:
                         max_axis_x = \
-                            occupancy_output['ground_plane'][tag].bounds[2]
+                            occupancy_output['ground_plane'].bounds[2]
                     else:
                         max_axis_x = max(
                             max_axis_x,
-                            occupancy_output['ground_plane'][tag].bounds[2])
+                            occupancy_output['ground_plane'].bounds[2])
 
                 if axis_y_limits is None:
                     if min_axis_y is None:
                         min_axis_y = \
-                            occupancy_output['ground_plane'][tag].bounds[1]
+                            occupancy_output['ground_plane'].bounds[1]
                     else:
                         min_axis_y = min(
                             min_axis_y,
-                            occupancy_output['ground_plane'][tag].bounds[1])
+                            occupancy_output['ground_plane'].bounds[1])
 
                     if max_axis_y is None:
                         max_axis_y = \
-                            occupancy_output['ground_plane'][tag].bounds[3]
+                            occupancy_output['ground_plane'].bounds[3]
                     else:
                         max_axis_y = max(
                             max_axis_y,
-                            occupancy_output['ground_plane'][tag].bounds[3])
+                            occupancy_output['ground_plane'].bounds[3])
+            else:
+                PCG_ROOT_LOGGER.info(
+                    'Ignoring ground plane from the occupancy grid')
+                ax.set_facecolor(free_color)
         else:
             PCG_ROOT_LOGGER.info(
                 'Ignoring ground plane from the occupancy grid')
@@ -642,11 +638,6 @@ def plot_occupancy_grid(
                 PCG_ROOT_LOGGER.info('Adding static models as occupied areas')
 
                 for tag in occupancy_output['static']:
-                    if is_excluded(tag):
-                        PCG_ROOT_LOGGER.info(
-                            'Model {} is excluded from final map'.format(tag))
-                        continue
-
                     if occupancy_output['static'][tag].is_empty:
                         PCG_ROOT_LOGGER.warning(
                             'Model footprint <{}> is empty'.format(tag))
@@ -701,11 +692,6 @@ def plot_occupancy_grid(
             PCG_ROOT_LOGGER.info('Adding non-static models as occupied areas')
 
             for tag in occupancy_output['non_static']:
-                if is_excluded(tag):
-                    PCG_ROOT_LOGGER.info(
-                        'Model {} is excluded from final map'.format(tag))
-                    continue
-
                 if occupancy_output['non_static'][tag].is_empty:
                     PCG_ROOT_LOGGER.warning(
                         'Model footprint <{}> is empty'.format(tag))
@@ -785,7 +771,7 @@ def plot_occupancy_grid(
 
     fig.canvas.draw()
 
-    if os.path.isdir(output_folder):
+    if output_folder is not None and os.path.isdir(output_folder):
         if '.pgm' in output_filename:
             filename = os.path.join(output_folder, output_filename)
             filename_svg = os.path.join(
@@ -793,7 +779,7 @@ def plot_occupancy_grid(
                     '.pgm', '.svg'))
             has_ground_plane = True
             if occupancy_output is not None:
-                has_ground_plane = len(occupancy_output['ground_plane']) > 0
+                has_ground_plane = occupancy_output['ground_plane'] is not None
 
             if with_ground_plane and has_ground_plane:
                 color = unavailable_color
